// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_RegularExpression_hpp
#define anna_core_util_RegularExpression_hpp


// STL
#include <string>

// C
#include <regex.h>

#include <anna/core/RuntimeException.hpp>


namespace anna {

/**
* Class helper to manage regular expressions with efficiency (first compile & keep pattern, then reuse match procedure with different values)
*/
class RegularExpression {

  std::string a_pattern;

  bool a_compiled;
  regex_t a_preg;


  void freeRegex() throw();
  void compile() throw(anna::RuntimeException);

public:

  RegularExpression(const std::string & pattern = "") : a_pattern(pattern), a_compiled(false) {};
  ~RegularExpression() { freeRegex(); }


  // set

  /**
  * Set the pattern for regular expression
  *
  * @param pattern Pattern
  */
  void setPattern(const std::string & pattern) throw();


  // get

  /**
  * Get the current pattern for the regular expression
  *
  * @return Pattern
  */
  const std::string & getPattern(void) const throw() { return a_pattern; }


  // helpers
  /**
  * Check if value fulfill regular expression
  *
  * @return Boolean about if value provided match regular expression
  */
  bool isLike(const std::string & value) throw();

  /**
  * Same as #isLike
  */
  bool match(const std::string & value) throw() { return isLike(value); }

  /**
  * Operator ==
  *
  * @param re Instance from RegularExpression class
  *
  * @return Returns re == this comparison based on private 'pattern' member
  */
  bool operator == (const RegularExpression & re) const;

  /**
  * Operator <
  *
  * @param re Instance from RegularExpression class
  *
  * @return Returns re < this comparison based on private 'pattern' member
  */
  bool operator < (const RegularExpression & re) const;
};

};


#endif
