// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_statistics_Meter_hpp
#define anna_statistics_Meter_hpp

#include <anna/timex/Meter.hpp>

// Standard
#include <string>

// Local
#include <anna/statistics/Accumulator.hpp>


namespace anna {
namespace xml {
class Node;
}
}


namespace anna {

namespace statistics {


using namespace anna;


//------------------------------------------------------------------------------
//------------------------------------------------------------------ class Meter
//------------------------------------------------------------------------------
/**
 * Accumulated data information for statistic sample oriented to processing time.
 * It manage a single concept, with integer nature sample and unit 'milliseconds'.
 *
 * @short Processing time control
 */
class Meter {
public:

  /**
  * Constructor.
  */
  Meter(const std::string & name);

  /**
  * Destructor.
  */
  ~Meter();


  /**
  * Sets the time control point for processing measure
  */
  void setControlPoint(void) throw() { a_meter.setControlPoint(); }


  /**
  * Reset sample
  */
  void reset(void) throw() { a_accumulator.reset(); }


  /**
  * Process new value for the sample: milliseconds time since last control point
  *
  * @return Returns processed delay
  */
  Millisecond process(void) throw() { a_accumulator.process(a_single_accumulator_concept_id, a_meter.getDelay()); return (a_meter.getDelay()); }


  /**
  * Copy operator
  *
  * @param Meter Class instance source of data
  *
  * @return Returns copied reference
  */
  const Meter & operator = (const Meter & meter);



  // Gets

//    /**
//    * Gets accumulator information in order to extract statistics indicators
//    */
//    const Accumulator & getAccumulator(void) const throw() { return (a_accumulator); }

  /**
  * Gets accumulator value information (statistics indicators)
  *
  * @param operation Solicited operation
  *
  * @return Value for solicited operation
  */
  double getAccumulatorValue(const Operation::Type &operation) const throw() { return (a_accumulator.getValue(a_single_accumulator_concept_id, operation)); }


  /**
  * Class string representation
  *
  * @param numberOfDecimals Number of float decimals at representation string. Default is 2.
  *
  * @return String with class content
  */
  std::string asString(const int & numberOfDecimals = 2) const throw();


  /**
  * Class XML representation
  *
  * @param numberOfDecimals Number of float decimals at XML representation. Default is 2.
  *
  * @return XML with class content
  */
  anna::xml::Node* asXML(anna::xml::Node* parent, const int & numberOfDecimals = 2) const throw();



private:

  timex::Meter a_meter;
  Accumulator a_accumulator;
  int a_single_accumulator_concept_id;
};

}
}

#endif
