// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_xml_DTD_hpp
#define anna_xml_DTD_hpp

struct _xmlDtd;
struct _xmlValidCtxt;
struct _xmlDoc;

#include <anna/core/mt/Mutex.hpp>
#include <anna/core/RuntimeException.hpp>

namespace anna {

namespace xml {

class Parser;

/**
   Clase base para manejar la DTD.

   A continuacion podemos ver un estracto de la DTD que definirias las reglas
   de formacion del documento XML mostrado como ejemplo en Parser.

   \code
   <?xml version="1.0" encoding="UTF-8"?>
   <!ELEMENT xvc (broadcast, ethernet, real, unreal)>
   <!ATTLIST xvc HeartBeat CDATA "2000">

   <!-- broadcast -->
   <!ELEMENT broadcast (INetAddress+)>
   <!ELEMENT INetAddress EMPTY>
   <!ATTLIST INetAddress Address CDATA #REQUIRED>
   <!ATTLIST INetAddress Port CDATA #REQUIRED>

   <!-- ethernet -->
   <!ELEMENT ethernet (Input, Output)>
   <!ATTLIST ethernet Mode (raw|ethernet) #REQUIRED>
   <!ATTLIST ethernet VirtualAddress CDATA #REQUIRED>
   <!ELEMENT Input EMPTY>
   <!ATTLIST Input Device CDATA #REQUIRED>
   <!ATTLIST Input PhysicalAccessPoint CDATA #REQUIRED>
   <!ATTLIST Input MAC CDATA #REQUIRED>
   <!ELEMENT Output EMPTY>
   <!ATTLIST Output Device CDATA #REQUIRED>
   <!ATTLIST Output PhysicalAccessPoint CDATA #REQUIRED>
   <!ATTLIST Output MAC CDATA #REQUIRED>
   \endcode
*/
class DTD {
public:
  /**
     Destructor.
  */
  virtual ~DTD();

  /**
     Inicializa la DTD con un nuevo contenido.
     \param content Contenido de la DTD, depedendiendo del tipo de DTD hara referencia
     a un nombre de archivo, una URI o a una cadena.
  */
  void initialize(const char* content) throw(RuntimeException);

protected:
  /**
     Constructor.
  */
  DTD() : a_handle(NULL) {;}

private:
  Mutex a_mutex;
  _xmlDtd* a_handle;

  void validate(_xmlValidCtxt* context, _xmlDoc* document) const throw(RuntimeException);
  virtual _xmlDtd* parse(const char* content) const throw(RuntimeException) = 0;

  friend class Parser;
};

}
}

#endif
