// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceMethod.hpp>

#include <anna/xml/Node.hpp>

#include <anna/comm/Communicator.hpp>
#include <anna/comm/Server.hpp>
#include <anna/comm/ClientSocket.hpp>
#include <anna/comm/Host.hpp>
#include <anna/comm/Device.hpp>
#include <anna/comm/functions.hpp>
#include <anna/comm/internal/RemoteConnection.hpp>

using namespace std;
using namespace anna;

comm::Server::Server(const string& name, const comm::Host& host, const int remotePort, const bool autoRecovery, comm::TransportFactory* transportFactory) :
  comm::Resource(name),
  a_host(host),
  a_remotePort(remotePort),
  a_autoRecovery(autoRecovery),
  a_transportFactory(transportFactory),
  a_clientSocket(NULL),
  a_msMaxConnectionDelay(ClientSocket::DefaultMaxConnectionDelay),
  a_msMaxWriteDelay(ClientSocket::DefaultMaxWriteDelay),
  a_receiverFactory(NULL),
  a_ignoreIncomingMessages(false),
  a_sequence(0)
  // La a_sequence se usa para diferenciar en las trazas y dems las conexiones que puede haber contra un mismo (IP, port)
  // Se establece desde comm::Host

{;}

//----------------------------------------------------------------------------------------
// (1) Para asegurar que el comm::handler::RemoteConnection no lo mete en la lista de
//     servidores pendientes de recuperar.
//----------------------------------------------------------------------------------------
comm::Server::~Server() {
  if(a_clientSocket == NULL)
    return;

  bool* autoRecovery = const_cast <bool*>(&a_autoRecovery);
  *autoRecovery = false;

  try {
    Communicator* communicator = functions::component <Communicator> (ANNA_FILE_LOCATION);
    communicator->detach(a_clientSocket);
  } catch(Exception& ex) {
    ex.trace();
  }
}

/*
 * Se invoca desde com::handle::RemoteConnection::finalize
 * [Tx] -> Communicator
 *
 * El socket se cierra protegido por esta SSCC para asegurar que los recursos se usan/bloquean en el mismo
 * orden que el usado en el Server::connect.
 */
void comm::Server::reset()
throw(RuntimeException) {
// La SSCC se establece en el mtodo que invoca a ste
//   Guard guard (*this, "comm::Server::reset");
  if(a_clientSocket == NULL)
    return;

  a_clientSocket->close();
  delete a_clientSocket;
  a_clientSocket = NULL;
}

void comm::Server::setReceiverFactory(comm::ReceiverFactory& receiverFactory)
throw() {
  a_receiverFactory = &receiverFactory;

  if(a_clientSocket != NULL)
    a_clientSocket->setReceiverFactory(receiverFactory);
}

/*
 * El punto conflictivo de llamada es desde el comm::ConnectionRecover::tryRecover
 *
 *    [T1] -> Communicator
 *
 * (1) La instancia RemoteConnection se libera en el handler::RemoteConnection::finalize.
 * (2) La espera maxima solo se aplica si ha sido modificada .
 * (3) La espera para la escritura sobre buffer de salida llenos solo se aplica si ha sido modificada.
 *
 * Lista de llamadas en 1.11.07
 *
[19/11/2009 13:22:07] Local7 | mt.db/Guard.cc (90) | thr: 0xb7819b20 | Guard::lock | Reference: (0x9f86090): comm::Host::createServer

   [19/11/2009 13:22:07] Information | comm.Host.cc (91) | thr: 0xb7819b20 | comm::Host::createServer | comm::Server { ...

   [19/11/2009 13:22:07] Local7 | mt.db/Guard.cc (90) | thr: 0xb7819b20 | Guard::lock | Reference: (0x9f865b0): comm::Server::connect

      [19/11/2009 13:22:07] Local7 | mt.db/Guard.cc (90) | thr: 0xb7819b20 | Guard::lock | Reference: (0xbfd67ea8): comm::Communicator::attach (RemoteConnection)

         [19/11/2009 13:22:07] Local7 | mt.db/Guard.cc (90) | thr: 0xb7819b20 | Guard::lock | Reference: (0x9f86608): anna::comm::ClientSocket (connect)

            [19/11/2009 13:22:07] Debug | comm.Socket.cc (171) | thr: 0xb7819b20 | anna::comm::Socket::open | comm::ClientSocket { comm::Socket { Domain: Inet | Type: Stream | comm::TransportFactory { Name: anna::comm::SureTransport | OverQuotaSize: 0 bytes } | fd: 4 | LocalPoint: { <null>  } | Bound: false } | RcvBufferSize: -1 bytes | Status: None | MaxConDelay: 200 ms | Reserved: 0 | Pending: 0 | Offset: 0 | ExpectedSize: -1 | Punto remoto: { comm::INetAddress { comm::Device { IP: 127.0.0.1 | Status: Up } | Port: 2000 }  } }

            [19/11/2009 13:22:07] Debug | comm.Socket.cc (219) | thr: 0xb7819b20 | anna::comm::Socket::setBlockingMode | Current: false | Previous: true

            [19/11/2009 13:22:07] Debug | comm.ClientSocket.cc (286) | thr: 0xb7819b20 | anna::comm::ClientSocket::connect | comm::ClientSocket { comm::Socket { Domain: Inet | Type: Stream | comm::TransportFactory { Name: anna::comm::SureTransport | OverQuotaSize: 0 bytes } | fd: 4 | LocalPoint: { <null>  } | Bound: false } | RcvBufferSize: 87380 bytes | Status: Connected | MaxConDelay: 200 ms | Reserved: 0 | Pending: 0 | Offset: 0 | ExpectedSize: -1 | Punto remoto: { comm::INetAddress { comm::Device { IP: 127.0.0.1 | Status: Up } | Port: 2000 }  } }

         [19/11/2009 13:22:07] Local7 | mt.db/Guard.cc (78) | thr: 0xb7819b20 | Guard::deactivate | Reference: (0x9f86608): anna::comm::ClientSocket (connect)

         [19/11/2009 13:22:07] Notice | comm.Communicator.cc (714) | thr: 0xb7819b20 | comm::Communicator::eventCreateConnection | comm::Server { anna::Resource { Nombre: 127.0.0.1:2000 | Habilitado: true | Disponible: true } | Auto-Recovery: false | comm::ClientSocket { comm::Socket { Domain: Inet | Type: Stream | comm::TransportFactory { Name: anna::comm::SureTransport | OverQuotaSize: 0 bytes } | fd: 4 | LocalPoint: { <null>  } | Bound: false } | RcvBufferSize: 87380 bytes | Status: Connected | MaxConDelay: 200 ms | Reserved: 0 | Pending: 0 | Offset: 0 | ExpectedSize: -1 | Punto remoto: { comm::INetAddress { comm::Device { IP: 127.0.0.1 | Status: Up } | Port: 2000 }  } } }

         [19/11/2009 13:22:07] Debug | comm.Communicator.cc (213) | thr: 0xb7819b20 | comm::Communicator::attach | comm::handler::RemoteConnection { comm::Handler { anna::Runnable { Id: Handler4 | Running: false | RequestedStop: false } | fd: 4 } | comm::RemoteConnection { comm::ClientSocket { comm::Socket { Domain: Inet | Type: Stream | comm::TransportFactory { Name: anna::comm::SureTransport | OverQuotaSize: 0 bytes } | fd: 4 | LocalPoint: { <null>  } | Bound: false } | RcvBufferSize: 87380 bytes | Status: Connected | MaxConDelay: 200 ms | Reserved: 0 | Pending: 0 | Offset: 0 | ExpectedSize: -1 | Punto remoto: { comm::INetAddress { comm::Device { IP: 127.0.0.1 | Status: Up } | Port: 2000 }  } } } }

      [19/11/2009 13:22:07] Local7 | mt.db/Guard.cc (78) | thr: 0xb7819b20 | Guard::deactivate | Reference: (0xbfd67ea8): comm::Communicator::attach (RemoteConnection)

      [19/11/2009 13:22:07] Debug | comm.Server.cc (163) | thr: 0xb7819b20 | anna::comm::Server::connect | comm::Server { anna::Resource { Nombre: 127.0.0.1:2000 | Habilitado: true | Disponible: true } | Auto-Recovery: false | comm::ClientSocket { comm::Socket { Domain: Inet | Type: Stream | comm::TransportFactory { Name: anna::comm::SureTransport | OverQuotaSize: 0 bytes } | fd: 4 | LocalPoint: { <null>  } | Bound: false } | RcvBufferSize: 87380 bytes | Status: Connected | MaxConDelay: 200 ms | Reserved: 0 | Pending: 0 | Offset: 0 | ExpectedSize: -1 | Punto remoto: { comm::INetAddress { comm::Device { IP: 127.0.0.1 | Status: Up } | Port: 2000 }  } } }

   [19/11/2009 13:22:07] Local7 | mt.db/Guard.cc (78) | thr: 0xb7819b20 | Guard::deactivate | Reference: (0x9f865b0): comm::Server::connect

[19/11/2009 13:22:07] Local7 | mt.db/Guard.cc (78) | thr: 0xb7819b20 | Guard::deactivate | Reference: (0x9f86090): comm::Host::createServer

*/
void comm::Server::connect()
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tm(Logger::Local7, "comm::Server", "connect", ANNA_FILE_LOCATION));
  Communicator* communicator = functions::component <Communicator> (ANNA_FILE_LOCATION);
  /*
   * Para asegurar el mismo orden de bloqueo a la hora de tratar la desconexin.
   */
  Guard guard(communicator, "comm::Communicator from comm::Server::connect");
  Guard guard2(*this, "comm::Server::connect");

  if(a_clientSocket != NULL && a_clientSocket->isConnected() == true)
    return;

  Host::const_device_iterator ii;
  Host::const_device_iterator maxii = a_host.device_end();
  int counter = 0;

  for(ii = a_host.device_begin(); ii != maxii; ii ++)
    counter += (Host::device(ii)->getStatus() == Device::Status::Up);

  if(counter > 0) {
    const Millisecond msMaxConnectionDelay = a_msMaxConnectionDelay / counter;
    RemoteConnection* remoteConnection = NULL;

    for(ii = a_host.device_begin(); ii != maxii && a_clientSocket == NULL; ii ++) {
      a_clientSocket = allocateClientSocket(INetAddress(Host::device(ii), a_remotePort), a_transportFactory);

      if(msMaxConnectionDelay != 0)
        a_clientSocket->setMaxConnectionDelay(msMaxConnectionDelay);            // (2)

      if(a_clientSocket->getMaxWriteDelay() != a_msMaxWriteDelay)
        a_clientSocket->setMaxWriteDelay(a_msMaxWriteDelay);                    // (3)

      if(a_receiverFactory != NULL)
        a_clientSocket->setReceiverFactory(*a_receiverFactory);

      a_clientSocket->setIgnoreIncomingMessages(a_ignoreIncomingMessages);
      remoteConnection = new RemoteConnection(this, a_clientSocket);             // (1)

      try {
        communicator->attach(remoteConnection);
      } catch(RuntimeException& ex) {
        delete a_clientSocket;
        delete remoteConnection;
        a_clientSocket = NULL;
        ex.trace();
      }
    }
  }

  if(a_clientSocket == NULL) {
    string msg(asString());
    msg += " | Cannot connect to server";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  LOGDEBUG(
    string msg("anna::comm::Server::connect | ");
    msg += asString();
    Logger::debug(msg, ANNA_FILE_LOCATION)
  );
}

//---------------------------------------------------------------------------------------
// (1) Mejora de la version 1.0.8 -> Si el Server no tiene reconexion automatica pero
// se intenta usar => se intenta volver a conectar antes de dar el fallo.
//---------------------------------------------------------------------------------------
comm::ClientSocket* comm::Server::send(Message& message)
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tm(Logger::Local7, "comm::Server", "send", ANNA_FILE_LOCATION));
  Guard guard(*this, "comm::Server::send");
  const bool available = isAvailable();
  const bool enabled = isEnabled();

  if(a_autoRecovery == false && available == false && enabled == true)     // (1)
    connect();

  if(available == false || enabled == false) {
    string msg(asString());
    msg += " | Server unavailable";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  a_clientSocket->send(message);
  return a_clientSocket;
}

comm::ClientSocket* comm::Server::send(Message* message)
throw(RuntimeException) {
  if(message == NULL)
    throw RuntimeException("anna::comm::Server::send | Cannot send a NULL message", ANNA_FILE_LOCATION);

  return send(*message);
}

void comm::Server::setAutoRecovery(bool autoRecovery) throw() {
  bool* ar = const_cast <bool*>(&a_autoRecovery);
  *ar = autoRecovery;
}

bool comm::Server::isAvailable() const
throw(RuntimeException) {
//   Guard guard (*this, "comm::Server::isAvailable");
  return (a_clientSocket == NULL) ? false : (a_clientSocket->isConnected() && (a_clientSocket->isClosedPending() == false));
}

//  Este metodo sea re-escrito en commsec::Server::allocateClientSocket para devolver un commsec::RemoteConnection
comm::ClientSocket* comm::Server::allocateClientSocket(const comm::INetAddress& in, comm::TransportFactory* transportFactory) const
throw() {
  return new ClientSocket(in, transportFactory);
}

string comm::Server::asString() const
throw() {
  string result("comm::Server { ");
  result += Resource::asString();
  result += " |  Sequence: ";
  result += functions::asString(a_sequence);
  result += " | Auto-Recovery: ";
  result += functions::asString(a_autoRecovery);
  result += " | ";

  if(a_clientSocket == NULL) {
    result += "Host: ";
    result += a_host.getName();
    result += functions::asString(" | RemotePort: %d", a_remotePort);
  } else
    result += a_clientSocket->asString();

  return result += " }";
}

xml::Node* comm::Server::asXML(xml::Node* parent) const
throw(RuntimeException) {
  xml::Node* result = parent->createChild("comm.Server");
  result->createAttribute("Host", a_host.getName());
  result->createAttribute("Sequence", a_sequence);
  comm::Resource::asAttribute(result);
  result->createAttribute("AutoRecovery", functions::asString(a_autoRecovery));
  result->createAttribute("RemotePort", a_remotePort);
  result->createAttribute("IgnoreIncomingMessages", functions::asString(a_ignoreIncomingMessages));

  if(a_clientSocket != NULL)
    result->createAttribute("fd", a_clientSocket->getfd());

  return result;
}



