// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/config/defines.hpp>
#include <anna/core/util/ComponentManager.hpp>
#include <anna/core/util/Component.hpp>

#include <anna/core/RuntimeException.hpp>
#include <anna/core/functions.hpp>
#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>
#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceMethod.hpp>


anna::Component* anna::ComponentManager::find(const char* className)
throw() {
  std::map <std::string, Component*>::iterator it = a_components.find(className);

  if(it != a_components.end()) return (*it).second;

  return NULL;
}

void anna::ComponentManager::attach(anna::Component* component)
throw(anna::RuntimeException) {
  LOGMETHOD(anna::TraceMethod tm("anna::ComponentManager", "attach(component)", ANNA_FILE_LOCATION));

  if(component == NULL)
    throw anna::RuntimeException("Cannot attach <null> component", ANNA_FILE_LOCATION);

  const char* className = component->getClassName();
  Component *already = find(className);

  if(already) {
    LOGINFORMATION(
      //std::string msg(already->asString());
      std::string msg(already->getClassName());
      msg += " | Was previously attached !";
      anna::Logger::information(msg, ANNA_FILE_LOCATION);
    )
    return;
  }

  LOGDEBUG(
    std::string msg("anna::ComponentManager::attach | ");
    msg += component->asString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  a_components[className] = component;
}

void anna::ComponentManager::detach(anna::Component* component)
throw(anna::RuntimeException) {
  LOGMETHOD(anna::TraceMethod tm("anna::ComponentManager", "detach(component)", ANNA_FILE_LOCATION));

  if(component == NULL)
    throw anna::RuntimeException("Cannot detach <null> component", ANNA_FILE_LOCATION);

  const char* className = component->getClassName();
  std::map <std::string, Component*>::iterator it = a_components.find(className);

  if(it == a_components.end()) {
    LOGINFORMATION(
      std::string msg(component->asString());
      msg += " | Not found: cannot erase !";
      anna::Logger::information(msg, ANNA_FILE_LOCATION);
    )
    return;
  }

  LOGDEBUG(
    std::string msg("anna::ComponentManager::detach | ");
    msg += component->asString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  a_components.erase(it);
}

anna::xml::Node* anna::ComponentManager::asXML(anna::xml::Node* parent) const
throw() {
  anna::xml::Node* node(NULL);
  node = parent->createChild("anna.Components");
  std::map <std::string, Component*>::const_iterator it;
  std::map <std::string, Component*>::const_iterator it_min(a_components.begin());
  std::map <std::string, Component*>::const_iterator it_max(a_components.end());
  Component *cc;

  for(it = it_min; it != it_max; it++) {
    anna::Guard guard(cc = (*it).second);
    cc->asXML(node);
  }

  return parent;
}


