// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/functions.hpp>

#include <anna/http/Transport.hpp>
#include <anna/http/Message.hpp>

#include <anna/http/parser/Abstract.hpp>
#include <anna/http/parser/WaitMessage.hpp>
#include <anna/http/parser/ReadHeader.hpp>
#include <anna/http/parser/WaitEndOfHeader.hpp>
#include <anna/http/parser/WaitChunkSize.hpp>
#include <anna/http/parser/ReadChunkSize.hpp>
#include <anna/http/parser/ReadChunkData.hpp>
#include <anna/http/parser/ReadChunkTrailers.hpp>

using namespace std;
using namespace anna;

namespace anna {
namespace http {
namespace parser {
WaitMessage st_waitMessage;
ReadHeader st_readHeader;
WaitEndOfHeader st_waitEndOfHeader;
WaitChunkSize st_waitChunkSize;
ReadChunkSize st_readChunkSize;
ReadChunkData st_readChunkData;
ReadChunkTrailers st_readChunkTrailers;
}
anna_assign_enum(parser::Abstract::ClassType) = {
  "WaitMessage", "ReadHeader", "WaitEndOfHeader",
  "WaitChunkSize", "ReadChunkSize", "ReadChunkData", "ReadChunkTrailers", NULL
};
}
}

string http::parser::Abstract::asString() const
throw() {
  string result("http::parser::Abstract { ");
  result += ClassType::asCString(a_classType);
  return result += " }";
}


/*static*/
void http::parser::Abstract::setState(http::Transport& transport, const http::parser::Abstract::ClassType::_v classType)
throw() {
  /*
   * Se repite el Wait-Message para llenar el hueco de ClassType::None
   */
  static Abstract* status [] =  {
    &st_waitMessage, &st_readHeader, &st_waitEndOfHeader,
    &st_waitChunkSize, &st_readChunkSize, &st_readChunkData, &st_readChunkTrailers
  };
  transport.setParserState(status [classType]);
}

/*static*/
void http::parser::Abstract::appendExtraParameter(http::Message* message, const std::string& extraParameter)
throw() {
  message->appendExtraParameter(extraParameter);
}

/*
 * Es acumulativo, porque cuando se invoca a
 * http::parser::ReadChunkData::processLine (http::Transport& transport, const DataBlock& dataBlock, const http::Token& line)
 * desde http::Transport::calculeSize no se pasa el dataBlock real (con el mensaje completo),
 * sino que se le pasa el trozo de chunk. Y el clculo del tamao de chunk se hace como diferencia de su ubicacin al
 * principio del mensaje;
 */
/*static*/
void http::parser::Abstract::setLastChunkedByte(http::Transport& transport, const int lastChunkedByte)
throw() {
  transport.a_lastChunkedByte += lastChunkedByte;
}

// Slo se puede usar en caso de Mensajes Transfer-encoding: chunked
/*static*/
const DataBlock& http::parser::Abstract::getFullMessage(http::Transport& transport)
throw() {
  return *transport.a_fullMessage;
}


