// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>

#include <anna/http/parser/ReadChunkData.hpp>

#include <anna/http/Transport.hpp>
#include <anna/http/functions.hpp>
#include <anna/http/internal/defines.hpp>
#include <anna/http/internal/EncodedBlock.hpp>

using namespace std;
using namespace anna;

/*
   Se ha comprobado que una vez que parace el Transfer-encoding puede haber un
   nmero indeterminado de header antes del chunk (propiemente dicho), pero
   siempre hay una lnea en blanco, que indica que la siguiente lnea es la que
   contiene el tamao del primer chunk, etc, etc

   160: 6e 65 63 74 69 6f 6e 3a 20 4b 65 65 70 2d 41 6c   nection: Keep-Al
   176: 69 76 65 0d 0a 54 72 61 6e 73 66 65 72 2d 45 6e   ive..Transfer-En
   192: 63 6f 64 69 6e 67 3a 20 63 68 75 6e 6b 65 64 0d   coding: chunked.
   208: 0a 43 6f 6e 74 65 6e 74 2d 54 79 70 65 3a 20 74   .Content-Type: t
   224: 65 78 74 2f 70 6c 61 69 6e 0d 0a 0d 0a 31 66 66   ext/plain....1ff     <================= ojo
   240: 38 0d 0a 3c 3f 78 6d 6c 20 76 65 72 73 69 6f 6e   8..<?xml version
   256: 3d 22 31 2e 30 22 20 65 6e 63 6f 64 69 6e 67 3d   ="1.0" encoding=

   464: 4a 61 6e 20 31 39 39 35 20 32 32 3a 30 30 3a 30   Jan 1995 22:00:0
   480: 30 20 47 4d 54 0d 0a 43 6f 6e 74 65 6e 74 2d 54   0 GMT..Content-T
   496: 79 70 65 3a 20 74 65 78 74 2f 68 74 6d 6c 0d 0a   ype: text/html..
   512: 54 72 61 6e 73 66 65 72 2d 65 6e 63 6f 64 69 6e   Transfer-encodin
   528: 67 3a 20 63 68 75 6e 6b 65 64 0d 0a 43 6f 6e 6e   g: chunked..Conn
   544: 65 63 74 69 6f 6e 3a 20 4b 65 65 70 2d 41 6c 69   ection: Keep-Ali
   560: 76 65 0d 0a 0d 0a 35 34 20 20 20 20 20 0d 0a 3c   ve....54     ..<     <================= ojo
   576: 21 2d 2d 20 6c 30 37 2e 6d 65 6d 62 65 72 2e 75   !-- l07.member.u
   592: 6b 6c 2e 79 61 68 6f 6f 2e 63 6f 6d 20 75 6e 63   kl.yahoo.com unc
   608: 6f 6d 70 72 65 73 73 65 64 2f 63 68 75 6e 6b 65   ompressed/chunke
   624: 64 20 57 65 64 20 53 65 70 20 20 33 20 31 30 3a   d Wed Sep  3 10:
   640: 31 34 3a 31 31 20 47 4d 54 20 32 30 30 38 20 2d   14:11 GMT 2008 -
   656: 2d 3e 0a 0d 0a 30 0d 0a 0d 0a                     ->...0....
*/
int http::parser::ReadChunkData::processLine(http::Transport& transport, const DataBlock& slice, const http::Token& line) const
throw(RuntimeException) {
  EncodedBlock* encodedBlock = transport.getEncodedBlock();

  if(encodedBlock->append(line) == EncodedBlock::State::Completed)
    setState(transport, ClassType::ReadChunkSize);
  else {
    // Marca el comienzo del chunk para que el http::Transport no lo vuelva a procesar
    // cuando llegen el resto de los trozos.
    const int chunkedByte = line.calculeOffset(slice) + line.getSize();
    parser::Abstract::setLastChunkedByte(transport, chunkedByte);
    LOGDEBUG(
      string msg("http::parser::ReadChunkData::processLine | ChunkByte: ");
      msg += anna::functions::asString(chunkedByte);
      Logger::debug(msg, ANNA_FILE_LOCATION);
    );
  }

  return -1;
}

